package com.starbuds.server.api.agent.common.service.admin;

import com.starbuds.server.api.agent.common.pojo.admin.AgentAdminInfoVO;
import com.starbuds.server.common.pojo.api.PageList;

/**
 * 管理员服务
 *
 * @author Lay
 * @date 2019/12/26
 */
public interface AgAdminService {
    /**
     * 获取管理员信息
     *
     * @param adminId
     * @return
     */
    AgentAdminInfoVO getInfo(Long adminId);

    /**
     * 判断管理员是否存在
     *
     * @param adminId 管理员id
     * @return 是否存在
     */
    boolean isAdminExist(Long adminId);


    /**
     * 创建管理员
     *
     * @param infoVO 管理员
     * @return 是否成功
     */
    boolean createAdmin(AgentAdminInfoVO infoVO);


    /**
     * 编辑管理员
     *
     * @param infoVO 管理员
     * @return 是否成功
     */
    boolean editAdmin(AgentAdminInfoVO infoVO);


    /**
     * 禁用管理员
     *
     * @param adminId 管理员id
     * @return 是否成功
     */
    boolean disableAdmin(Long adminId);

    /**
     * 启用管理员
     *
     * @param adminId 管理员id
     * @return 是否成功
     */
    boolean enableAdmin(Long adminId);

    /**
     * 删除管理员
     *
     * @param adminId 管理员id
     * @return 是否成功
     */
    boolean deleteAdmin(Long adminId);

    /**
     * 获取管理员列表
     *
     * @param adminName  管理员姓名
     * @param adminPhone 手机号
     * @param status     状态
     * @param pageNum    当前页码
     * @param pageSize   分页大小
     * @param agentId    公会id
     * @return result对象
     */
    PageList<AgentAdminInfoVO> getAdminPageList(String adminName, String adminPhone, Integer status, int pageNum, int pageSize, Long agentId);

    /**
     * 判断是否存在账号
     *
     * @param adminPhone 手机号
     * @return 是否存在
     */
    boolean isPhoneExist(String adminPhone);


    /**
     * 是否可用
     *
     * @param adminId
     * @return
     */
    boolean isAdminEnable(Long adminId);

    /**
     * 获取公会id
     *
     * @param adminId
     * @return
     */
    Long getAgentId(Long adminId);
}
