package com.starbuds.server.api.agent.common.service.admin;

import com.starbuds.server.api.agent.common.pojo.admin.AnchorAuditVO;
import com.starbuds.server.api.agent.common.pojo.admin.AnchorVO;
import com.starbuds.server.api.agent.common.pojo.admin.LiveStreamHisVO;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.media.LiveHisVO;

import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/4/22 17:28
 * @Description 主播业务
 */
public interface AgAnchorService {

    /**
     * 获取主播列表
     *
     * @param userName     用户姓名
     * @param agentId      公会id
     * @param startDate    开始时间
     * @param endDate      结束时间
     * @param liveStatus   直播状态
     * @param streamStatus 推流状态
     * @param userNo       星芽号
     * @param userPhone    手机号
     * @param charmLevel   主播等级
     * @param pageNum      当前页码
     * @param pageSize     分页大小
     * @param orderField   排序字段
     * @param isAsc        是否升序
     * @return 分页对象
     */
    PageList<AnchorVO> getAnchorPageList(String userName,
                                         Long agentId,
                                         Long startDate,
                                         Long endDate,
                                         Integer liveStatus,
                                         Integer streamStatus,
                                         Long userNo,
                                         String userPhone,
                                         Integer charmLevel,
                                         int pageNum,
                                         int pageSize,
                                         String orderField,
                                         boolean isAsc);


    /**
     * 直播记录列表
     *
     * @param userId
     * @param pageIndex
     * @param pageSize
     * @return
     */
    PageList<LiveHisVO> getLiveHisPageList(Long userId, int pageIndex, int pageSize);

    /**
     * 获取主播详情
     *
     * @param userId 用户id
     * @return 主播详情
     */
    AnchorVO getAnchorDetail(Long userId);

    /**
     * 获取主播认证记录
     *
     * @param userId 用户id
     * @return 认证记录
     */
    List<AnchorAuditVO> getAnchorAuditRecord(Long userId);

    /**
     * 获取公户id
     *
     * @param userId
     * @return
     */
    Long getAgentId(Long userId);

    /**
     * 获取推流记录
     *
     * @param liveHisId
     * @return
     */
    List<LiveStreamHisVO> getLiveStreamHis(Long liveHisId);

}
