package com.starbuds.server.api.agent.extension;

import com.starbuds.server.api.agent.constant.HttpError;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.NotAllowedException;
import javax.ws.rs.NotFoundException;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Response;
import javax.ws.rs.ext.ExceptionMapper;
import javax.ws.rs.ext.Provider;

/**
 * 自定义异常捕获
 *
 * @author Lay
 * @date 2019-02-18
 */
@Slf4j
@Component
@Provider
@Produces(ContentType.APPLICATION_JSON_UTF_8)
public class CustomExceptionMapper implements ExceptionMapper<Exception> {

    @Override
    public Response toResponse(Exception e) {
        log.error("访问出错,", e);
        if (e instanceof NotFoundException) {
            log.error("无效的接口路径");
            return HttpError.NOT_FOUND;
        } else if (e instanceof NumberFormatException || e instanceof javax.ws.rs.BadRequestException) {
            log.error("无效的参数");
            return HttpError.BAD_REQUEST;
        } else if (e instanceof NotAllowedException) {
            log.error("无效的请求方式");
            return HttpError.METHOD_NOT_ALLOWED;
        } else {
            log.error("处理错误", e);
            return HttpError.SERVER_ERROR;
        }
    }
}