package com.starbuds.server.api.agent.service.admin.dao;

import com.starbuds.server.api.agent.common.pojo.admin.AgentAdminInfoVO;
import com.starbuds.server.api.agent.common.pojo.db.AgentAdminDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020-01-15 14:12:01
 * @Description 家族管理员持久层
 */
@Mapper
@Repository
public interface AgentAdminDAO {
    /**
     * 新增
     *
     * @param entity
     * @return
     */
    int insert(AgentAdminDO entity);

    /**
     * 根据主键更新
     *
     * @param entity
     * @return
     */
    int updateById(AgentAdminDO entity);

    /**
     * 根据主键查询
     *
     * @param adminId
     * @return
     */
    AgentAdminDO selectById(@Param("adminId") Long adminId);

    /**
     * 根据id获取vo
     *
     * @param adminId
     * @return
     */
    AgentAdminInfoVO selectVOById(@Param("adminId") Long adminId);


    /**
     * 查询adminId是否存在
     *
     * @param adminId
     * @return
     */
    int countById(@Param("adminId") Long adminId);

    /**
     * 更新状态
     *
     * @param adminId
     * @param adminStatus
     * @return
     */
    int updateStatus(@Param("adminId") Long adminId, @Param("adminStatus") Integer adminStatus);

    /**
     * 获取管理员列表
     *
     * @param adminName   管理员姓名
     * @param adminPhone  售价好
     * @param adminStatus 状态
     * @param pageCursor  光标
     * @param pageSize    分页大小
     * @param agentId     公会id
     * @return 管理员列表
     */
    List<AgentAdminInfoVO> getAdminPageList(@Param("adminName") String adminName,
                                            @Param("adminPhone") String adminPhone,
                                            @Param("adminStatus") Integer adminStatus,
                                            @Param("pageCursor") int pageCursor,
                                            @Param("pageSize") int pageSize,
                                            @Param("agentId") Long agentId);

    /**
     * 获取管理员列表大小
     *
     * @param adminName   管理员姓名
     * @param adminPhone  售价好
     * @param adminStatus 状态
     * @param agentId     公会id
     * @return 列表大小
     */
    int countAdminPageList(@Param("adminName") String adminName,
                           @Param("adminPhone") String adminPhone,
                           @Param("adminStatus") Integer adminStatus,
                           @Param("agentId") Long agentId);


    /**
     * 获取管理员状态
     *
     * @param adminId
     * @return
     */
    Integer getAdminStatus(@Param("adminId") Long adminId);

    /**
     * 获取公会id
     *
     * @param adminId
     * @return
     */
    Long selectAgentId(@Param("adminId") Long adminId);
}
