package com.starbuds.server.api.agent.service.admin.dao;

import com.starbuds.server.api.agent.common.pojo.admin.AnchorVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/27 16:17
 * @Description 主播持久层
 */
@Mapper
@Repository
public interface AnchorDAO {

    /**
     * 更新公会id
     *
     * @param userId
     * @param agentId
     * @return
     */
    int updateAgentId(@Param("userId") Long userId, @Param("agentId") Long agentId);

    /**
     * 获取主播列表
     *
     * @param userName     用户姓名
     * @param agentId      公户id
     * @param startDate    开始时间
     * @param endDate      结束时间
     * @param liveStatus   直播状态
     * @param streamStatus 推流状态
     * @param userNo       星芽号
     * @param userPhone    手机号
     * @param charmLevel   主播等级
     * @param pageCursor   光标
     * @param pageSize     分页大小
     * @param orderField   排序字段
     * @param sort         升序、降序
     * @return
     */
    List<AnchorVO> getAnchorPageList(@Param("userName") String userName,
                                     @Param("agentId") Long agentId,
                                     @Param("startDate") Long startDate,
                                     @Param("endDate") Long endDate,
                                     @Param("liveStatus") Integer liveStatus,
                                     @Param("streamStatus") Integer streamStatus,
                                     @Param("userNo") Long userNo,
                                     @Param("userPhone") String userPhone,
                                     @Param("charmLevel") Integer charmLevel,
                                     @Param("pageCursor") int pageCursor,
                                     @Param("pageSize") int pageSize,
                                     @Param("orderField") String orderField,
                                     @Param("sort") String sort);

    /**
     * 获取主播列表大小
     *
     * @param userName     用户姓名
     * @param agentId      公会id
     * @param startDate    开始时间
     * @param endDate      结束时间
     * @param liveStatus   直播状态
     * @param streamStatus 推流状态
     * @param userNo       星芽号
     * @param userPhone    手机号
     * @param charmLevel   主播等级
     * @return 分页大小
     */
    int countAnchorPageList(@Param("userName") String userName,
                            @Param("agentId") Long agentId,
                            @Param("startDate") Long startDate,
                            @Param("endDate") Long endDate,
                            @Param("liveStatus") Integer liveStatus,
                            @Param("streamStatus") Integer streamStatus,
                            @Param("userNo") Long userNo,
                            @Param("userPhone") String userPhone,
                            @Param("charmLevel") Integer charmLevel);


    /**
     * 获取主播详情
     *
     * @param userId
     * @return
     */
    AnchorVO getAnchorDetail(@Param("userId") Long userId);

    /**
     * 获取公会id
     *
     * @param userId
     * @return
     */
    Long selectAgentId(@Param("userId") Long userId);
}