package com.starbuds.server.api.agent.service.admin.impl;

import com.starbuds.server.api.agent.common.pojo.admin.AgentBankcardVO;
import com.starbuds.server.api.agent.common.service.admin.AgAgentBankcardService;
import com.starbuds.server.api.agent.service.admin.dao.AgentBankcardDAO;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.AgentBankcardDO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/3/30 15:02
 * @Description 公会银行卡业务实现
 */
@Service
public class AgAgentBankcardServiceImpl implements AgAgentBankcardService {

    @Resource
    AgentBankcardDAO agentBankcardDAO;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createAgentBankcard(AgentBankcardDO bankcardDO) {
        return agentBankcardDAO.insert(bankcardDO) == 1;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean editAgentBankcard(AgentBankcardDO bankcardDO) {
        return agentBankcardDAO.updateById(bankcardDO) == 1;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean disableAgentBankcard(Long cardId) {
        return agentBankcardDAO.updateStatus(cardId, CommonStatus.Disabled) == 1;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean enableAgentBankcard(Long cardId) {
        return agentBankcardDAO.updateStatus(cardId, CommonStatus.Normal) == 1;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean deleteAgentBankcard(Long cardId) {
        return agentBankcardDAO.updateStatus(cardId, CommonStatus.Deleted) == 1;
    }

    @Override
    public boolean isAgentBankcardExist(Long cardId) {
        return agentBankcardDAO.countById(cardId) > 0;
    }

    @Override
    public AgentBankcardVO getAgentBankcardDetail(Long cardId) {
        return agentBankcardDAO.getDetail(cardId);
    }

    @Override
    public PageList<AgentBankcardVO> getAgentBankcardPageList(Long agentId, String cardNo, Integer cardStatus, Integer cardType, int pageIndex, int pageSize) {
        PageList<AgentBankcardVO> page = new PageList<>(pageIndex, pageSize);
        page.setList(agentBankcardDAO.getAgentBankcardPageList(agentId, cardNo, cardStatus, cardType, page.getPageCursor(), pageSize));
        page.setTotal(agentBankcardDAO.countAgentBankcardPageList(agentId, cardNo, cardStatus, cardType));
        return page;
    }

    @Override
    public Integer getCardType(Long cardId) {
        return agentBankcardDAO.selectCardType(cardId);
    }
}
