package com.starbuds.server.api.agent.service.admin.impl;

import com.starbuds.server.api.agent.common.service.admin.AgAgentService;
import com.starbuds.server.api.agent.service.admin.dao.*;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.user.ApplyTypeEnum;
import com.starbuds.server.common.constant.user.AuditStatusEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.AgentAnchorDO;
import com.starbuds.server.common.pojo.db.AgentAnchorHisDO;
import com.starbuds.server.common.pojo.db.AgentAuditDO;
import com.starbuds.server.common.pojo.user.AgentAuditVO;
import com.starbuds.server.common.pojo.user.AgentVO;
import com.starbuds.server.common.service.user.SocialRelationService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/1/15 14:23
 * @Description 公会审核业务实现
 */
@Slf4j
@Service
public class AgAgentServiceImpl implements AgAgentService {

    @Autowired
    AgentAuditDAO agentAuditDAO;

    @Autowired
    AgentAnchorDAO agentAnchorDAO;

    @Autowired
    AgentAnchorHisDAO agentAnchorHisDAO;

    @Autowired
    AnchorDAO anchorDAO;

    @Autowired
    AgentDAO agentDAO;

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean passAudit(Long auditId, Long adminId, String auditMsg, Double anchorRatio) {

        AgentAuditDO dbAudit = agentAuditDAO.selectById(auditId);
        if (dbAudit == null) {
            log.error("未找到申请记录");
            return false;
        }

        long currentTime = System.currentTimeMillis();

        AgentAuditDO auditDO = new AgentAuditDO();
        auditDO.setAuditId(auditId);
        auditDO.setAdminId(adminId);
        auditDO.setAuditMsg(auditMsg);
        auditDO.setAuditStatus(AuditStatusEnum.PASS.getCode());
        auditDO.setAuditTime(currentTime);
        agentAuditDAO.updateById(auditDO);


        ApplyTypeEnum typeEnum = ApplyTypeEnum.toEnum(dbAudit.getApplyType());
        // 写入签约记录
        AgentAnchorHisDO hisDO = new AgentAnchorHisDO();
        switch (typeEnum) {
            case JOIN:
                // 创建签约关系
                AgentAnchorDO anchorDO = new AgentAnchorDO();
                anchorDO.setUserId(dbAudit.getUserId());
                anchorDO.setAgentId(dbAudit.getAgentId());
                anchorDO.setSignTime(currentTime);
                anchorDO.setEndTime(-1L);
                anchorDO.setStatus(CommonStatus.Normal);
                if (anchorRatio == null || anchorRatio > 1 || anchorRatio < 0) {
                    anchorDO.setRatio(agentDAO.selectAnchorRatio(dbAudit.getAgentId()));
                } else {
                    anchorDO.setRatio(anchorRatio);
                }
                agentAnchorDAO.insert(anchorDO);

                // 冗余公会id
                anchorDAO.updateAgentId(dbAudit.getUserId(), dbAudit.getAgentId());

                // 写入签约记录
                hisDO.setType(dbAudit.getApplyType());
                hisDO.setAgentId(dbAudit.getAgentId());
                hisDO.setCreateTime(currentTime);
                hisDO.setUserId(dbAudit.getUserId());
                return agentAnchorHisDAO.insert(hisDO) == 1;
            case LEAVE:

                Long signId = agentAnchorDAO.selectSignId(dbAudit.getAgentId(), dbAudit.getUserId());

                anchorDAO.updateAgentId(dbAudit.getUserId(), null);

                agentAnchorDAO.updateStatusAndEndTime(signId, CommonStatus.Disabled, currentTime);

                hisDO.setType(dbAudit.getApplyType());
                hisDO.setAgentId(dbAudit.getAgentId());
                hisDO.setCreateTime(currentTime);
                hisDO.setUserId(dbAudit.getUserId());
                return agentAnchorHisDAO.insert(hisDO) == 1;
            default:
                break;
        }
        return false;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean denyAudit(Long auditId, Long adminId, String auditMsg) {
        AgentAuditDO auditDO = new AgentAuditDO();
        auditDO.setAuditId(auditId);
        auditDO.setAdminId(adminId);
        auditDO.setAuditMsg(auditMsg);
        auditDO.setAuditStatus(AuditStatusEnum.DENY.getCode());
        auditDO.setAuditTime(System.currentTimeMillis());
        return agentAuditDAO.updateById(auditDO) == 1;
    }

    @Override
    public boolean isAuditExist(Long auditId) {
        return agentAuditDAO.countById(auditId) == 1;
    }

    @Override
    public PageList<AgentAuditVO> getAuditPageList(Long agentId, Integer applyType, Long userId, Integer auditStatus, String userName, int pageNum, int pageSize) {
        PageList<AgentAuditVO> page = new PageList<>(pageNum, pageSize);
        List<AgentAuditVO> auditList = agentAuditDAO.getAuditPageList(agentId, applyType, userId, auditStatus, userName, page.getPageCursor(), pageSize);
        if (!CollectionUtils.isEmpty(auditList)) {
            for (AgentAuditVO vo : auditList) {
                vo.setFansCount(socialRelationService.getFansCount(vo.getUserId()));
                vo.setAnchorRatio(agentAnchorDAO.selectAnchorRatio(vo.getAgentId(), vo.getUserId()));
                vo.setSignId(agentAnchorDAO.selectSignId(vo.getAgentId(), vo.getUserId()));
            }
        }
        page.setList(auditList);
        page.setTotal(agentAuditDAO.countAuditPageList(agentId, applyType, userId, auditStatus, userName));
        return page;
    }

    /**
     * 获取公会详情
     *
     * @param agentId 公会id
     * @return 公会详情
     */
    @Override
    public AgentVO getAgentDetail(Long agentId) {
        AgentVO vo = agentDAO.getAgentDetail(agentId);
        if (vo != null) {
            vo.setAnchorCount(agentAnchorDAO.getAnchorCount(agentId));
        }
        return vo;
    }

    /**
     * 公会是否存在
     *
     * @param agentId 公会id
     * @return 是否存在
     */
    @Override
    public boolean isAgentExist(Long agentId) {
        return agentDAO.countById(agentId) > 0;
    }

    @Override
    public Double getAnchorRatio(Long agentId) {
        return agentDAO.selectAnchorRatio(agentId);
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean setAnchorRatio(Long signId, Double ratio) {
        return agentAnchorDAO.setAnchorRatio(signId, ratio) == 1;
    }

    @Override
    public boolean isSignExist(Long signId) {
        return agentAnchorDAO.countById(signId) > 0;
    }

    @Override
    public Integer getAuditStatus(Long auditId) {
        return agentAuditDAO.selectAuditStatus(auditId);
    }
}
