package com.starbuds.server.api.agent.service.admin.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.api.agent.common.pojo.admin.AgentWalletWithdrawHisVO;
import com.starbuds.server.api.agent.common.service.admin.AgAgentWalletWithdrawHisService;
import com.starbuds.server.api.agent.service.admin.dao.*;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.AgentWalletDO;
import com.starbuds.server.common.pojo.db.AgentWalletHisDO;
import com.starbuds.server.common.pojo.db.AgentWalletWithdrawHisDO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/3/30 16:19
 * @Description 公会钱包提现记录业务实现
 */
@Slf4j
@Service
public class AgAgentWalletWithdrawHisServiceImpl implements AgAgentWalletWithdrawHisService {

    @Resource
    AgentWalletWithdrawHisDAO agentWalletWithdrawHisDAO;

    @Resource
    AgentWalletDAO agentWalletDAO;

    @Resource
    AgentWalletHisDAO agentWalletHisDAO;

    @Resource
    AgentBankcardDAO agentBankcardDAO;

    @Resource
    AgentDAO agentDAO;

    @Resource
    AdminDAO adminDAO;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BigDecimal addWithdrawApply(AgentWalletWithdrawHisDO withdrawHisDO, String hisRemarks) {
        try {
            // 公会id
            Long agentId = withdrawHisDO.getAgentId();
            // 检查账户
            AgentWalletDO walletDO = checkWallet(agentId);
            // 提现金额
            BigDecimal balanceChange = withdrawHisDO.getAmount();
            // 原有余额
            BigDecimal balanceOld = walletDO.getBalance();
            // 原有版本号
            Long version = walletDO.getVersion();
            // 更新余额
            BigDecimal balanceNew = updateBalance(agentId, balanceOld, version, balanceChange);
            if (balanceNew.compareTo(BigDecimal.ZERO) == -1) {
                throw new RuntimeException("余额不足");
            }
            // 写入记录
            if (agentWalletWithdrawHisDAO.insert(withdrawHisDO) == 1) {
                AgentWalletHisDO walletHisDO = new AgentWalletHisDO();
                walletHisDO.setHisId(withdrawHisDO.getHisId());
                walletHisDO.setAgentId(agentId);
                walletHisDO.setBalanceOld(walletDO.getBalance());
                walletHisDO.setBalanceValue(withdrawHisDO.getAmount());
                walletHisDO.setBalanceNew(balanceNew);
                walletHisDO.setCreateTime(System.currentTimeMillis());
                walletHisDO.setHisRemarks(hisRemarks);
                walletHisDO.setOrderId(withdrawHisDO.getHisId());
                agentWalletHisDAO.insert(walletHisDO);
                return balanceNew;
            }
        } catch (Exception e) {
            log.info("更新失败{}", e);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        return null;
    }

    @Override
    public BigDecimal getAmount(Long agentId) {
        return Optional.ofNullable(agentWalletDAO.selectBalance(agentId)).orElse(BigDecimal.ZERO);
    }

    @Override
    public Integer getApplyStatus(Long agentId) {
        return agentWalletWithdrawHisDAO.selectLastApplyStatus(agentId);
    }

    @Override
    public PageList<AgentWalletWithdrawHisVO> getAgentWithdrawHisPageList(Long agentId, Integer applyStatus, Long startTime, Long endTime, int pageIndex, int pageSize) {
        PageList<AgentWalletWithdrawHisVO> page = new PageList<>(pageIndex, pageSize);
        List<AgentWalletWithdrawHisVO> list = agentWalletWithdrawHisDAO.getAgentWithdrawHisPageList(agentId, applyStatus, startTime, endTime, page.getPageCursor(), pageSize);
        if (!CollectionUtil.isEmpty(list)) {
            for (AgentWalletWithdrawHisVO vo : list) {
                vo.setAgentName(agentDAO.selectAgentName(vo.getAgentId()));
                vo.setBankcard(agentBankcardDAO.selectById(vo.getBankcardId()));
                vo.setApplyAdminName(adminDAO.selectAdminName(vo.getApplyAdminId()));
            }
        }
        page.setList(list);
        page.setTotal(agentWalletWithdrawHisDAO.countAgentWithdrawHisPageList(agentId, applyStatus, startTime, endTime));
        return page;
    }


    /**
     * 更新余额
     *
     * @param agentId
     * @param balance
     * @param version
     * @param changeBalance
     * @return
     */
    private BigDecimal updateBalance(Long agentId, BigDecimal balance, Long version, BigDecimal changeBalance) {
        AgentWalletDO walletUpdate = new AgentWalletDO();
        walletUpdate.setAgentId(agentId);
        walletUpdate.setUpdateTime(System.currentTimeMillis());
        walletUpdate.setBalance(balance.add(changeBalance.negate()));
        walletUpdate.setVersion(version);
        // 扣钱CAS
        if (agentWalletDAO.updateBalanceCas(walletUpdate) == 0) {
            AgentWalletDO walletDO = agentWalletDAO.selectById(agentId);
            return updateBalance(agentId, walletDO.getBalance(), walletDO.getVersion(), changeBalance);
        }
        return walletUpdate.getBalance();
    }

    /**
     * 检查账户
     *
     * @param agentId
     */
    private AgentWalletDO checkWallet(Long agentId) {
        // 判断账户是否存在
        AgentWalletDO agentWalletDO = agentWalletDAO.selectById(agentId);
        // 无则初始化
        if (agentWalletDO == null) {
            agentWalletDO = new AgentWalletDO();
            agentWalletDO.setAgentId(agentId);
            agentWalletDO.setBalance(BigDecimal.ZERO);
            agentWalletDO.setVersion(0L);
            agentWalletDO.setUpdateTime(System.currentTimeMillis());
            try {
                agentWalletDAO.insert(agentWalletDO);
            } catch (Exception e) {
                checkWallet(agentId);
            }
        }
        return agentWalletDO;
    }
}
