package com.starbuds.server.api.agent.service.admin.impl;

import com.starbuds.server.api.agent.common.pojo.admin.AnchorAuditVO;
import com.starbuds.server.api.agent.common.pojo.admin.AnchorVO;
import com.starbuds.server.api.agent.common.pojo.admin.LiveStreamHisVO;
import com.starbuds.server.api.agent.common.service.admin.AgAnchorService;
import com.starbuds.server.api.agent.service.admin.dao.*;
import com.starbuds.server.common.constant.finance.ScoreTypeEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.media.LiveHisVO;
import com.starbuds.server.common.service.user.AnchorService;
import com.starbuds.server.common.service.user.ScoreService;
import com.starbuds.server.common.service.user.SocialRelationService;
import com.starbuds.server.common.service.user.UserService;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Optional;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/4/22 17:29
 * @Description 主播业务
 */
@Service
public class AgAnchorServiceImpl implements AgAnchorService {

    @Resource
    AnchorDAO anchorDAO;

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    ScoreService scoreService;

    @Resource
    AdminDAO adminDAO;

    @Resource
    LiveHisDAO liveHisDAO;

    @Resource
    SingleChatCategoryDAO singleChatCategoryDAO;

    @Resource
    AnchorAuditDAO anchorAuditDAO;

    @Resource
    LiveStreamHisDAO liveStreamHisDAO;

    @Override
    public PageList<AnchorVO> getAnchorPageList(String userName, Long agentId, Long startDate, Long endDate, Integer liveStatus, Integer streamStatus, Long userNo, String userPhone, Integer charmLevel, int pageNum, int pageSize, String orderField, boolean isAsc) {
        PageList<AnchorVO> page = new PageList<>(pageNum, pageSize);
        List<AnchorVO> list = anchorDAO.getAnchorPageList(userName, agentId, startDate, endDate, liveStatus, streamStatus, userNo, userPhone, charmLevel, page.getPageCursor(), pageSize, orderField, isAsc ? "ASC" : "DESC");
        if (!CollectionUtils.isEmpty(list)) {
            for (AnchorVO vo : list) {
                vo.setFansCount(socialRelationService.getFansCount(vo.getUserId()));
                vo.setDisable(anchorService.isDisable(vo.getUserId()));
                vo.setDisableAdminName(vo.getDisableAdminId() == null ? "" : adminDAO.selectAdminName(vo.getDisableAdminId()));
                vo.setAuthAdminName(vo.getAuthAdminId() == null ? "" : adminDAO.selectAdminName(vo.getAuthAdminId()));
                vo.setCharmValue(scoreService.getScoreValue(vo.getUserId(), ScoreTypeEnum.CHARM));
            }
        }
        page.setList(list);
        page.setTotal(anchorDAO.countAnchorPageList(userName, agentId, startDate, endDate, liveStatus, streamStatus, userNo, userPhone, charmLevel));
        return page;
    }

    @Override
    public PageList<LiveHisVO> getLiveHisPageList(Long userId, int pageIndex, int pageSize) {
        PageList<LiveHisVO> page = new PageList<>(pageIndex, pageSize);
        page.setList(liveHisDAO.getLiveHisPageList(userId, page.getPageCursor(), pageSize));
        page.setTotal(liveHisDAO.countLiveHisPageList(userId));
        return page;
    }

    /**
     * 获取主播详情
     *
     * @param userId 用户id
     * @return 主播详情
     */
    @Override
    public AnchorVO getAnchorDetail(Long userId) {
        AnchorVO anchorVO = anchorDAO.getAnchorDetail(userId);
        if (anchorVO != null) {
            anchorVO.setUserProfile(Optional.ofNullable(userService.getProfile(userId)).orElse(null));
            anchorVO.setDisable(anchorService.isDisable(userId));
            anchorVO.setFansCount(socialRelationService.getFansCount(userId));
            anchorVO.setDisable(anchorService.isDisable(userId));
            anchorVO.setDisableAdminName(anchorVO.getDisableAdminId() == null ? "" : adminDAO.selectAdminName(anchorVO.getDisableAdminId()));
            anchorVO.setAuthAdminName(anchorVO.getAuthAdminId() == null ? "" : adminDAO.selectAdminName(anchorVO.getAuthAdminId()));
            anchorVO.setSingleChatCategoryName(singleChatCategoryDAO.selectCategoryName(anchorVO.getSingleChatCategoryId()));
            anchorVO.setCharmValue(scoreService.getScoreValue(anchorVO.getUserId(), ScoreTypeEnum.CHARM));
        }
        return anchorVO;
    }

    /**
     * 获取主播认证记录
     *
     * @param userId 用户id
     * @return 认证记录
     */
    @Override
    public List<AnchorAuditVO> getAnchorAuditRecord(Long userId) {
        return anchorAuditDAO.getAnchorAuditRecord(userId);
    }

    @Override
    public Long getAgentId(Long userId) {
        return anchorDAO.selectAgentId(userId);
    }

    @Override
    public List<LiveStreamHisVO> getLiveStreamHis(Long liveHisId) {
        return liveStreamHisDAO.getLiveStreamHis(liveHisId);
    }
}
