package com.starbuds.server.api.agent.v1.facade;

import com.starbuds.server.common.pojo.api.Result;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/1/15 19:06
 * @Description 公会接口
 */
public interface AnchorFacade {

    /**
     * 通过
     *
     * @param auditId
     * @param auditMsg
     * @param anchorRatio
     * @return
     */
    Result passAudit(Long auditId, String auditMsg, Double anchorRatio);

    /**
     * 拒绝
     *
     * @param auditId
     * @param auditMsg
     * @return
     */
    Result denyAudit(Long auditId, String auditMsg);

    /**
     * 获取主播列表
     *
     * @param userName     用户姓名
     * @param startDate    开始时间
     * @param endDate      结束时间
     * @param liveStatus   直播状态
     * @param streamStatus 推流状态
     * @param userNo       星芽号
     * @param userPhone    手机号
     * @param charmLevel   主播等级
     * @param pageNum      当前页码
     * @param pageSize     分页大小
     * @param orderField   排序字段
     * @param isAsc        是否升序
     * @return result对象
     */
    Result getAnchorPageList(String userName,
                             Long startDate,
                             Long endDate,
                             Integer liveStatus,
                             Integer streamStatus,
                             Long userNo,
                             String userPhone,
                             Integer charmLevel,
                             int pageNum,
                             int pageSize,
                             String orderField,
                             boolean isAsc);

    /**
     * 获取入会申请列表
     *
     * @param agentId
     * @param applyType
     * @param userId
     * @param auditStatus
     * @param userName
     * @param pageNum
     * @param pageSize
     * @return
     */
    Result getAuditPageList(Long agentId, Integer applyType, Long userId, Integer auditStatus, String userName, int pageNum, int pageSize);

    /**
     * 获取公会详情
     *
     * @param agentId
     * @return
     */
    Result getAgentDetail(Long agentId);

    /**
     * 获取公会主播分成比例
     *
     * @param agentId
     * @return
     */
    Result getAnchorRatio(Long agentId);

    /**
     * 设置主播分成比例
     *
     * @param signId
     * @param ratio
     * @return
     */
    Result setAnchorRatio(Long signId, Double ratio);

    /**
     * 直播记录列表
     *
     * @param userId
     * @param pageIndex
     * @param pageSize
     * @return
     */
    Result getLiveHisPageList(Long userId, int pageIndex, int pageSize);

    /**
     * 获取主播详情
     *
     * @param userId 用户id
     * @return result对象
     */
    Result getAnchorDetail(Long userId);

    /**
     * 获取主播认证记录
     *
     * @param userId 用户id
     * @return result对象
     */
    Result getAnchorAuditRecord(Long userId);

    /**
     * 获取推流记录
     *
     * @param liveHisId
     * @return
     */
    Result getLiveStreamHis(Long liveHisId);

}
