package com.starbuds.server.api.agent.v1.facade;

import com.starbuds.server.api.agent.common.pojo.admin.AgentBankcardReq;
import com.starbuds.server.common.pojo.api.Result;

import java.math.BigDecimal;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/3/30 15:15
 * @Description 财务管理
 */
public interface FinanceFacade {

    /**
     * 提现账户-创建卡号
     *
     * @param bankcardReq
     * @return
     */
    Result createAgentBankcard(AgentBankcardReq bankcardReq);

    /**
     * 提现账户-编辑卡号
     *
     * @param bankcardReq
     * @return
     */
    Result editAgentBankcard(AgentBankcardReq bankcardReq);

    /**
     * 提现账户-禁用卡号
     *
     * @param cardId
     * @return
     */
    Result disableAgentBankcard(Long cardId);

    /**
     * 提现账户-启用卡号
     *
     * @param cardId
     * @return
     */
    Result enableAgentBankcard(Long cardId);

    /**
     * 提现账户-删除卡号
     *
     * @param cardId
     * @return
     */
    Result deleteAgentBankcard(Long cardId);

    /**
     * 提现账户-获取详情
     *
     * @param cardId
     * @return
     */
    Result getAgentBankcardDetail(Long cardId);

    /**
     * 提现账户-获取列表
     *
     * @param cardNo
     * @param cardStatus
     * @param cardType
     * @param pageIndex
     * @param pageSize
     * @return
     */
    Result getAgentBankcardPageList(String cardNo, Integer cardStatus, Integer cardType, int pageIndex, int pageSize);

    /**
     * 申请提现
     *
     * @param amount
     * @param cardId
     * @return
     */
    Result applyWithdraw(BigDecimal amount, Long cardId);

    /**
     * 获取余额
     *
     * @return
     */
    Result getBalance();

    /**
     * 获取公会提现申请列表
     *
     * @param applyStatus
     * @param startTime
     * @param endTime
     * @param pageIndex
     * @param pageSize
     * @return
     */
    Result getAgentWithdrawHisPageList(Integer applyStatus, Long startTime, Long endTime, int pageIndex, int pageSize);

}
