package com.starbuds.server.api.agent.v1.impl;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.api.agent.common.pojo.admin.TokenParam;
import com.starbuds.server.common.pojo.api.HeaderPackage;
import org.apache.commons.lang3.StringUtils;
import org.jboss.resteasy.spi.ResteasyProviderFactory;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.math.BigInteger;

/**
 * 接口基类
 *
 * @author Lay
 * @date 2019-02-18
 */
public class BaseFacade {
    protected HttpServletRequest getRequest() {
        return ResteasyProviderFactory.getInstance().getContextData(HttpServletRequest.class);
    }

    protected HttpServletResponse getResponse() {
        return ResteasyProviderFactory.getInstance().getContextData(HttpServletResponse.class);
    }

    protected String getParamString(String name) {
        return this.getRequest().getParameter(name);
    }

    protected int getParamInt(String name) {
        return Convert.toInt(this.getParamString(name));
    }

    protected float getParamFloat(String name) {
        return Convert.toFloat(this.getParamString(name));
    }

    protected double getParamDouble(String name) {
        return Convert.toDouble(this.getParamString(name));
    }

    protected long getParamLong(String name) {
        return Convert.toLong(this.getParamString(name));
    }

    protected String getHeaderString(String name) {
        String str = this.getRequest().getHeader(name);
        return StringUtils.isBlank(str) ? "" : str;
    }

    protected int getHeaderInt(String name) {
        return Convert.toInt(this.getHeaderString(name));
    }

    protected BigInteger getHeaderBigInteger(String name) {
        String str = this.getHeaderString(name);
        return str == "" ? BigInteger.ZERO : new BigInteger(str);
    }

    protected float getHeaderFloat(String name) {
        return Convert.toFloat(this.getHeaderString(name));
    }

    protected double getHeaderDouble(String name) {
        return Convert.toDouble(this.getHeaderString(name));
    }

    protected long getHeaderLong(String name) {
        return Convert.toLong(this.getHeaderString(name));
    }

    protected HeaderPackage getHeader() {
        Object headerPackage = this.getRequest().getAttribute("header");
        if (headerPackage != null) {
            return (HeaderPackage) headerPackage;
        }
        return null;
    }

    protected TokenParam getToken() {
        Object token = this.getRequest().getAttribute("token");
        if (token != null) {
            return (TokenParam) token;
        }
        return null;
    }
}
