package com.starbuds.server.api.agent.v1.impl;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.api.agent.common.pojo.admin.AgentBankcardReq;
import com.starbuds.server.api.agent.common.service.admin.AgAdminService;
import com.starbuds.server.api.agent.common.service.admin.AgAgentBankcardService;
import com.starbuds.server.api.agent.common.service.admin.AgAgentWalletService;
import com.starbuds.server.api.agent.common.service.admin.AgAgentWalletWithdrawHisService;
import com.starbuds.server.api.agent.v1.facade.FinanceFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.CardTypeEnum;
import com.starbuds.server.common.constant.finance.WalletWithdrawHisApplyStatus;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.AgentBankcardDO;
import com.starbuds.server.common.pojo.db.AgentWalletWithdrawHisDO;
import com.starbuds.server.common.service.core.IdentityService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.*;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Optional;

/**
 * 财务管理
 *
 * @author jiangxiaowei
 * @date 2020/3/30 15:16
 */
@Path("/v1/finance")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class FinanceFacadeImpl extends BaseFacade implements FinanceFacade {

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Resource
    AgAgentBankcardService agAgentBankcardService;

    @Resource
    AgAdminService agAdminService;

    @Resource
    AgAgentWalletService agAgentWalletService;

    @Resource
    AgAgentWalletWithdrawHisService agAgentWalletWithdrawHisService;

    /**
     * 提现账户-创建卡号
     *
     * @param bankcardReq
     * @return
     */
    @POST
    @Path("createAgentBankcard")
    @Override
    public Result createAgentBankcard(@BeanParam AgentBankcardReq bankcardReq) {
        AgentBankcardDO bankcardDO = new AgentBankcardDO();
        BeanUtils.copyProperties(bankcardReq, bankcardDO);
        bankcardDO.setCardId(identityService.getId());
        bankcardDO.setCardCreateTime(System.currentTimeMillis());
        bankcardDO.setAgentId(this.getToken().getAgentId());

        if (agAgentBankcardService.createAgentBankcard(bankcardDO)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 提现账户-编辑卡号
     *
     * @param bankcardReq
     * @return
     */
    @POST
    @Path("editAgentBankcard")
    @Override
    public Result editAgentBankcard(@BeanParam AgentBankcardReq bankcardReq) {
        if (!agAgentBankcardService.isAgentBankcardExist(bankcardReq.getCardId())) {
            return new Result(GlobalApiError.OperationFailed, "卡号不存在");
        }
        AgentBankcardDO bankcardDO = new AgentBankcardDO();
        BeanUtils.copyProperties(bankcardReq, bankcardDO);
        if (agAgentBankcardService.editAgentBankcard(bankcardDO)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 提现账户-禁用卡号
     *
     * @param cardId
     * @return
     */
    @POST
    @Path("disableAgentBankcard")
    @Override
    public Result disableAgentBankcard(@FormParam("cardId") Long cardId) {
        if (!agAgentBankcardService.isAgentBankcardExist(cardId)) {
            return new Result(GlobalApiError.OperationFailed, "卡号不存在");
        }
        if (agAgentBankcardService.disableAgentBankcard(cardId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 提现账户-启用卡号
     *
     * @param cardId
     * @return
     */
    @POST
    @Path("enableAgentBankcard")
    @Override
    public Result enableAgentBankcard(@FormParam("cardId") Long cardId) {
        if (!agAgentBankcardService.isAgentBankcardExist(cardId)) {
            return new Result(GlobalApiError.OperationFailed, "卡号不存在");
        }
        if (agAgentBankcardService.enableAgentBankcard(cardId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 提现账户-删除卡号
     *
     * @param cardId
     * @return
     */
    @POST
    @Path("deleteAgentBankcard")
    @Override
    public Result deleteAgentBankcard(@FormParam("cardId") Long cardId) {
        if (!agAgentBankcardService.isAgentBankcardExist(cardId)) {
            return new Result(GlobalApiError.OperationFailed, "卡号不存在");
        }
        if (agAgentBankcardService.deleteAgentBankcard(cardId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 提现账户-获取详情
     *
     * @param cardId
     * @return
     */
    @GET
    @Path("getAgentBankcardDetail")
    @Override
    public Result getAgentBankcardDetail(@QueryParam("cardId") Long cardId) {
        return new Result(agAgentBankcardService.getAgentBankcardDetail(cardId));
    }

    /**
     * 提现账户-获取列表
     *
     * @param cardNo
     * @param cardStatus
     * @param cardType
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @GET
    @Path("getAgentBankcardPageList")
    @Override
    public Result getAgentBankcardPageList(@QueryParam("cardNo") String cardNo,
                                           @QueryParam("cardStatus") Integer cardStatus,
                                           @QueryParam("cardType") Integer cardType,
                                           @QueryParam("pageIndex") int pageIndex,
                                           @QueryParam("pageSize") int pageSize) {
        return new Result(agAgentBankcardService.getAgentBankcardPageList(this.getToken().getAgentId(), cardNo, cardStatus, cardType, pageIndex, pageSize));
    }

    /**
     * 申请提现
     *
     * @param amount
     * @param cardId
     * @return
     */
    @POST
    @Path("applyWithdraw")
    @Override
    public Result applyWithdraw(@FormParam("amount") BigDecimal amount, @FormParam("cardId") Long cardId) {

        Long agentId = this.getToken().getAgentId();

        // 判断余额是否足够
        if (agAgentWalletService.getBalance(agentId).compareTo(amount) == -1) {
            return new Result(GlobalApiError.OperationFailed, "余额不足");
        }

        Integer cardType = agAgentBankcardService.getCardType(cardId);
        if (cardType == null) {
            return new Result(GlobalApiError.OperationFailed, "卡号不存在");
        }

        AgentWalletWithdrawHisDO withdrawHisDO = new AgentWalletWithdrawHisDO();
        withdrawHisDO.setHisId(identityService.getId());
        withdrawHisDO.setAmount(amount);
        withdrawHisDO.setApplyStatus(WalletWithdrawHisApplyStatus.WAIT.getValue());
        withdrawHisDO.setBankcardId(cardId);
        withdrawHisDO.setHisCreateTime(System.currentTimeMillis());
        withdrawHisDO.setAgentId(agentId);
        withdrawHisDO.setUserTaxRatio(cardType == CardTypeEnum.PRIVATE.getValue() ? 0.06d : null);
        withdrawHisDO.setTaxAmount(withdrawHisDO.getUserTaxRatio() != null ? withdrawHisDO.getAmount().multiply(new BigDecimal(withdrawHisDO.getUserTaxRatio())).setScale(2, RoundingMode.UP) : null);
        withdrawHisDO.setActualAmount(amount.subtract(Optional.ofNullable(withdrawHisDO.getTaxAmount()).orElse(BigDecimal.ZERO).setScale(2, RoundingMode.DOWN)));
        withdrawHisDO.setAgentAmount(withdrawHisDO.getActualAmount());
        withdrawHisDO.setWalletId(Convert.toStr(agentId));

        BigDecimal newBalance = agAgentWalletWithdrawHisService.addWithdrawApply(withdrawHisDO, "提现转出");

        if (newBalance == null) {
            return new Result(GlobalApiError.OperationFailed, "余额不足");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("balance", newBalance);
        return mResult.build();
    }

    /**
     * 获取余额
     *
     * @return
     */
    @GET
    @Path("getBalance")
    @Override
    public Result getBalance() {
        Long agentId = this.getToken().getAgentId();
        MutableResult mResult = new MutableResult();
        mResult.put("agentId", agentId);
        mResult.put("balance", agAgentWalletWithdrawHisService.getAmount(agentId));
        mResult.put("applyStatus", agAgentWalletWithdrawHisService.getApplyStatus(agentId));
        return mResult.build();
    }

    /**
     * 获取公会提现申请列表
     *
     * @param applyStatus
     * @param startTime
     * @param endTime
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @GET
    @Path("getAgentWithdrawHisPageList")
    @Override
    public Result getAgentWithdrawHisPageList(@QueryParam("applyStatus") Integer applyStatus,
                                              @QueryParam("startTime") Long startTime,
                                              @QueryParam("endTime") Long endTime,
                                              @QueryParam("pageIndex") int pageIndex,
                                              @QueryParam("pageSize") int pageSize) {
        return new Result(agAgentWalletWithdrawHisService.getAgentWithdrawHisPageList(this.getToken().getAgentId(), applyStatus, startTime, endTime, pageIndex, pageSize));
    }
}
